pub struct PeakEnvelopeFollower { /* private fields */ }Expand description
Peak envelope follower with independent attack/release time constants, as used for audio dynamics processing (compressors, limiters, VU-style level meters).
Implementations
Sourceimpl PeakEnvelopeFollower
impl PeakEnvelopeFollower
Sourcepub fn new(attack_samples: f32, release_samples: f32) -> Self
pub fn new(attack_samples: f32, release_samples: f32) -> Self
attack_samples / release_samples: the time constant, in samples, for the envelope
to rise / fall 1 - 1/e (~63%) of the way to a step change in input level.