pub struct Chain<A, B> {
pub first: A,
pub second: B,
}Expand description
A sequential composition of two DSP nodes A and B with zero runtime overhead.
Fields
first: Asecond: BTrait Implementations
impl<A: Copy, B: Copy> Copy for Chain<A, B>
Sourceimpl<T: Copy, A: DspNode<T>, B: DspNode<T>> DspNode<T> for Chain<A, B>
impl<T: Copy, A: DspNode<T>, B: DspNode<T>> DspNode<T> for Chain<A, B>
Sourcefn process_sample(&mut self, input: T) -> T
fn process_sample(&mut self, input: T) -> T
Process a single input sample and produce one output sample.
Sourcefn process_block(&mut self, in_buf: &[T], out_buf: &mut [T])
fn process_block(&mut self, in_buf: &[T], out_buf: &mut [T])
Process a block of samples from
in_buf into out_buf.Sourcefn process_in_place(&mut self, buf: &mut [T])
fn process_in_place(&mut self, buf: &mut [T])
Process a block of samples in place.