Module spatial

Module spatial 

2D Spatial and Image Processing routines (2D DCT/IDCT, 2D Convolution, 2D Nonlinear Filters, Sobel Edge Detection, 2D Histogram, MSE/PSNR).

Enums

NonlinFilterType
Non-linear 2D spatial filter type.

Functions

convolve2d_f32
Performs 2D spatial convolution of src image (rows x cols) with a k_rows x k_cols kernel.
dct2d_f32
Computes the 2D Discrete Cosine Transform (DCT-II) on a rows x cols image.
histogram_2d_f32
Computes the histogram of a 2D image / matrix into bins spanning [min_val, max_val].
idct2d_f32
Computes the 2D Inverse Discrete Cosine Transform (IDCT-II) on a rows x cols coefficient matrix.
mse_2d_f32
Computes the Mean Squared Error (MSE) between two 2D images.
nonlin2d_filter_f32
Performs 2D non-linear filtering (Min, Max, or Median) on a rows x cols image using a k_size x k_size square window.
psnr_2d_f32
Computes the Peak Signal-to-Noise Ratio (PSNR) in dB between two 2D images.
sobel_edge_detection_f32
Applies Sobel edge detection to a 2D image, outputting gradient magnitude and binary edge detection.