Matrix operations (addition, subtraction, multiplication, scale, transpose, inverse, complex matrix multiplication).
Structs
- Matrix
Instance - Matrix structure wrapping a slice of data in row-major order.
- Matrix
Instance Mut - Mutable Matrix structure wrapping a mutable slice of data in row-major order.
Functions
- mat_
add_ f32 - mat_
add_ q15 - mat_
add_ q31 - mat_
inverse_ f32 - mat_
mult_ f32 - mat_
mult_ q15 - mat_
mult_ q31 - mat_
scale_ f32 - mat_
scale_ q15 - mat_
scale_ q31 - mat_
sub_ f32 - mat_
sub_ q15 - mat_
sub_ q31 - mat_
trans_ f32 - mat_
trans_ q15 - mat_
trans_ q31 - polynomial_
eval_ f32 - Evaluates a polynomial $P(x) = c_0 + c_1 x + c_2 x^2 + \dots + c_d x^d$ using Horner’s method.
- polynomial_
least_ squares_ fit - Fits a polynomial of degree
degree($y = c_0 + c_1 x + c_2 x^2 + \dots + c_d x^d$) to data points $(x_i, y_i)$ using weighted linear least-squares regression.