Module matrix

Module matrix 

Matrix operations (addition, subtraction, multiplication, scale, transpose, inverse, complex matrix multiplication).

Structs

MatrixInstance
Matrix structure wrapping a slice of data in row-major order.
MatrixInstanceMut
Mutable Matrix structure wrapping a mutable slice of data in row-major order.

Functions

mat_add_f32
mat_add_q15
mat_add_q31
mat_inverse_f32
mat_mult_f32
mat_mult_q15
mat_mult_q31
mat_scale_f32
mat_scale_q15
mat_scale_q31
mat_sub_f32
mat_sub_q15
mat_sub_q31
mat_trans_f32
mat_trans_q15
mat_trans_q31