pub fn convolve2d_f32(
src: &[f32],
dst: &mut [f32],
rows: usize,
cols: usize,
kernel: &[f32],
k_rows: usize,
k_cols: usize,
normalize: bool,
) -> StatusExpand description
Performs 2D spatial convolution of src image (rows x cols) with a k_rows x k_cols kernel.
If normalize is true, the convolution output is divided by the sum of the absolute kernel weights.